import 'dart:async';

import 'package:flutter/services.dart';

import '../MethodName.dart';
import '../WitsystemSDK.dart';
import '../command/CardType.dart';
import '../command/FaceType.dart';
import '../command/MarkType.dart';
import '../command/PasswordType.dart';
import 'dart:convert' as convert;

/// 设备的基本蓝牙操作

class BleDevice extends WisystemSDk {
  StreamSubscription? _addMarkListen;

  final Stream<Map<String, dynamic>> _bleAddMarkEvents =
      const EventChannel(ChannelName.bleAddMark)
          .receiveBroadcastStream()
          .map((event) => convert.json.decode(event.toString()));

  factory BleDevice() => _getInstance();

  static BleDevice get instance => _getInstance();
  static BleDevice? _instance;

  BleDevice._internal();

  static BleDevice _getInstance() {
    _instance ??= BleDevice._internal();
    return _instance!;
  }

  /// 通过蓝牙添加密码
  addPassword(String deviceId, PasswordType passwordType, String password,
      String passwordName,
      {String? userId}) async {
    return await methodChannel.invokeMethod("addPassword", {
      "deviceId": deviceId,
      "passwordType": passwordType.name,
      "password": password,
      "passwordName": passwordName,
      "userId": userId
    });
  }

  /// 通过蓝牙添加密码
  deletePassword(String deviceId, String uid) async {
    return await methodChannel
        .invokeMethod("deletePassword", {"deviceId": deviceId, "uid": uid});
  }

  /// 通过蓝牙添加卡
  addCard(String deviceId, CardType cardType, String cardName,
      {String? userId}) async {
    return await methodChannel.invokeMethod("addCard", {
      "deviceId": deviceId,
      "cardType": cardType.name,
      "cardName": cardName,
      "userId": userId
    });
  }

  /// 取消蓝牙添加卡
  cancelAddCard(String deviceId) async {
    return await methodChannel
        .invokeMethod("cancelAddCard", {"deviceId": deviceId});
  }

  /// 通过蓝牙添加密码
  deleteCard(String deviceId, String uid) async {
    return await methodChannel
        .invokeMethod("deletePassword", {"deviceId": deviceId, "uid": uid});
  }

  /// 通过蓝牙冻结或者解冻卡
  freezeCard(String deviceId, String uid, bool isFreeze) async {
    return await methodChannel.invokeMethod(
        "freezeCard", {"deviceId": deviceId, "uid": uid, "isFreeze": isFreeze});
  }

  /// 通过蓝牙开启或者关闭刷卡功能
  closeOrOpenCard(String deviceId) async {
    return await methodChannel
        .invokeMethod("closeOrOpenCard", {"deviceId": deviceId});
  }

  /// 通过蓝牙添加指纹
  addMark(
    String deviceId,
    MarkType markType,
    String markName, {
    String? userId,
    Function()? sendSuccess,
    Function(Map map)? commandError,
    Function(Map map)? fail,
    Function(Map map)? progress,
    Function(Map map)? success,
  }) {
    _addMarkListen = _bleAddMarkEvents.listen((event) {
      switch (event["function"]) {
        case "progress":
          if (progress != null) progress(event["data"]);
          break;
        case "sendSuccess":
          if (sendSuccess != null) sendSuccess();
          break;
        case "success":
          if (success != null) success(event["data"]);
          _addMarkListen!.cancel();
          break;
      }
    });
    _addMarkListen!.onError((error) {
      var error2 = error as PlatformException;
      switch (error2.code) {
        case "commandError":
          if (commandError != null) {
            commandError({"code": error2.message, "error": error2.details});
          }
          break;
        case "fail":
          if (fail != null) {
            fail({"code": error2.message, "error": error2.details});
          }
          break;
        default:
          print('未知的错误$error');
          break;
      }
      _addMarkListen!.cancel();
    });

    methodChannel.invokeMethod("addMark", {
      "deviceId": deviceId,
      "markType": markType.name,
      "markName": markName,
      "userId": userId
    });
  }

  /// 通过蓝牙添加指纹
  cancelAddMark(String deviceId) async {
    return await methodChannel
        .invokeMethod("cancelAddMark", {"deviceId": deviceId});
  }

  /// 通过蓝牙删除指纹
  deleteMark(String deviceId, String uid) async {
    return await methodChannel
        .invokeMethod("deleteMark", {"deviceId": deviceId, "uid": uid});
  }

  /// 通过蓝牙冻结或者解冻指纹
  freezeMark(String deviceId, String uid, bool isFreeze) async {
    return await methodChannel.invokeMethod(
        "freezeCard", {"deviceId": deviceId, "uid": uid, "isFreeze": isFreeze});
  }

  /// 通过蓝牙添加人脸
  addFace(String deviceId, FaceType faceType, String faceName,
      {String? userId}) async {
    return await methodChannel.invokeMethod("addFace", {
      "deviceId": deviceId,
      "faceType": faceType.name,
      "faceName": faceName,
      "userId": userId
    });
  }

  /// 通过蓝牙添加人脸
  cancelAddFace(String deviceId) async {
    return await methodChannel
        .invokeMethod("cancelAddFace", {"deviceId": deviceId});
  }

  /// 通过蓝牙删除人脸
  deleteFace(String deviceId, String uid) async {
    return await methodChannel
        .invokeMethod("deleteFace", {"deviceId": deviceId, "uid": uid});
  }

  /// 通过蓝牙冻结或者解冻人脸
  freezeFace(String deviceId, String uid, bool isFreeze) async {
    return await methodChannel.invokeMethod(
        "freezeFace", {"deviceId": deviceId, "uid": uid, "isFreeze": isFreeze});
  }

  /// 通过蓝牙修改音量
  updateVolume(String deviceId, int volume) async {
    return await methodChannel
        .invokeMethod("updateVolume", {"deviceId": deviceId, "volume": volume});
  }

  /// 通过蓝牙更新设备状态
  updateDeviceState(String deviceId) async {
    return await methodChannel
        .invokeMethod("updateDeviceState", {"deviceId": deviceId});
  }

  /// 通过关闭指定设备的蓝牙操作
  closeBleDevice(String deviceId) async {
    return await methodChannel
        .invokeMethod("closeBleDevice", {"deviceId": deviceId});
  }



}
